/*
 * EdgePanel.java
 *
 * Author: Pieter Deelen
 * Created: November 29, 2005, 11:30 AM
 *
 */

package tracevis.visualization.detail;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JTable;

import tracevis.model.ProgramInterface;
import tracevis.model.types.CallData;
import tracevis.model.types.ClassData;
import tracevis.utilities.TableSorter;
import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Vertex;


/**
 * A panel which shows information about an edge.
 * @author  Pieter Deelen
 */
public class EdgePanel extends javax.swing.JPanel {
	private final CallData callData;
	private final ClassData sourceClassData;
	private final ClassData destClassData;

	private class EdgeMethodsTableModel extends MethodsTableModel {
		private final List<String> methodNames;

		public EdgeMethodsTableModel() {
			methodNames = new ArrayList<String>();

			for (String method : callData.getMethods()) {
				methodNames.add(method);
			}
		}

		@Override
		protected int getMethodCount() {
			return methodNames.size();
		}

		@Override
		protected String getMethodName(int rowIndex) {
			return methodNames.get(rowIndex);
		}

		@Override
		protected int getMethodCallCount(int rowIndex) {
			String methodName = methodNames.get(rowIndex);
			return callData.getMethodCallCount(methodName);
		}
	}

	private final MethodsTableModel methodsTableModel;

	/**
	 * Creates new form EdgePanel.
	 * @param edge the edge to visualize.
	 */
	public EdgePanel(ProgramInterface program, DirectedEdge edge) {
		callData = (CallData)edge.getUserDatum("tracevis.model.Program");

		Vertex sourceVertex = edge.getSource();
		sourceClassData = (ClassData)sourceVertex.getUserDatum("tracevis.model.Program");

		Vertex destVertex = edge.getDest();
		destClassData = (ClassData)destVertex.getUserDatum("tracevis.model.Program");

		methodsTableModel = new EdgeMethodsTableModel();

		initComponents();
		update();
	}

	public void update() {
		totalValueLabel.setText(Integer.toString(callData.getCallCount()));
	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        fromKeyLabel = new javax.swing.JLabel();
        fromValueLabel = new javax.swing.JLabel();
        toKeyLabel = new javax.swing.JLabel();
        toValueLabel = new javax.swing.JLabel();
        totalKeyLabel = new javax.swing.JLabel();
        totalValueLabel = new javax.swing.JLabel();
        methodsTablePanel = new javax.swing.JPanel();
        TableSorter tableSorter = new TableSorter(methodsTableModel);
        methodsTable = new JTable(tableSorter);
        tableSorter.setTableHeader(methodsTable.getTableHeader());
        tableSorter.setSortingStatus(0, TableSorter.ASCENDING);
        dummyPanel = new javax.swing.JPanel();

        setLayout(new java.awt.GridBagLayout());

        setBorder(javax.swing.BorderFactory.createTitledBorder("Calls"));
        fromKeyLabel.setText("From:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        add(fromKeyLabel, gridBagConstraints);

        fromValueLabel.setText(sourceClassData.getShortName());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        add(fromValueLabel, gridBagConstraints);

        toKeyLabel.setText("To:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        add(toKeyLabel, gridBagConstraints);

        toValueLabel.setText(destClassData.getShortName());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        add(toValueLabel, gridBagConstraints);

        totalKeyLabel.setText("Total:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        add(totalKeyLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        add(totalValueLabel, gridBagConstraints);

        methodsTablePanel.setLayout(new java.awt.BorderLayout());

        methodsTablePanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Methods:"));
        methodsTablePanel.add(methodsTable.getTableHeader(), BorderLayout.NORTH);
        methodsTablePanel.add(methodsTable, java.awt.BorderLayout.CENTER);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        add(methodsTablePanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(dummyPanel, gridBagConstraints);

    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel dummyPanel;
    private javax.swing.JLabel fromKeyLabel;
    private javax.swing.JLabel fromValueLabel;
    private javax.swing.JTable methodsTable;
    private javax.swing.JPanel methodsTablePanel;
    private javax.swing.JLabel toKeyLabel;
    private javax.swing.JLabel toValueLabel;
    private javax.swing.JLabel totalKeyLabel;
    private javax.swing.JLabel totalValueLabel;
    // End of variables declaration//GEN-END:variables

}
